unit MannDoor;

interface

uses
    Classes, Crt, EleNorm, MannAnsi, MannUtls, SysUtils, vpSysLow, vpUtils;

type
  TKeyTypes = (ktLocal, ktRemote);

  TUserInfo =           Record
    Access:             LongInt;
    Baud:               LongInt;
    ComNum:             LongInt;
    ComType:            LongInt;
    Handle:             String[20];
    IdleMax:            LongInt;
    IdleTime:           LongInt;
    RecNum:             LongInt;
    TimeLeft:           LongInt;
    TimeMax:            LongInt;
    TimeOn:             LongInt;
    UserF:              String[20];
    UserL:              String[20];
  end;

var
   KeyType: TKeyTypes;
   isExtended: Boolean;
   isLocal: Boolean;
   mUserInfo: TUserInfo;
   LastKeyPress: LongInt;

   DropCarrierProc: Procedure;
   LocalLoginProc: Procedure;
   StatusBarProc: Procedure;
   TimeOutProc: procedure;
   TimeUpProc: Procedure;

function mCarrier: Boolean;
procedure mClearBuf;
procedure mClose;
procedure mClrEol;
procedure mClrScr;
procedure mCrlf;
procedure mDisplayFile(S: String);
procedure mGotoXY(X, Y: Byte);
function mInput(S, Mask: String; ShowLen, MaxLen, Attr: Byte): String;
function mInputLn(S, Mask: String; ShowLen, MaxLen, Attr: Byte): String;
function mKeyPressed: Boolean;
procedure mOpen(ComNum, ComType: LongInt);
function mReadKey: Char;
procedure mStartUp(DropFile: String; Local: Boolean);
procedure mTextBackground(BG: Byte);
procedure mTextColor(FG: Byte);
procedure mWrite(S: String);
procedure mWriteLn(S: String);

procedure DefaultDropCarrierProc;
procedure DefaultLocalLoginProc;
procedure DefaultStatusBarProc;
procedure DefaultTimeOutProc;
procedure DefaultTimeUpProc;

implementation

{$I DROPFILE.INC} { Contains procedures to read dropfiles }
{$I PROCS.INC} { Contains procedures to check for events }

function HandleProcs(P: Pointer): LongInt;
var
   CurTime: LongInt;
begin
     repeat
           SysCtrlSleep(500);
           CurTime := SecondsSinceMidnight;
           CheckDropCarrier;
           CheckStatusBar;
           CheckTimeIdle(CurTime, LastKeyPress);
           CheckTimeLeft(CurTime);
     until (1=2);
end;

procedure mClose;
begin
     if Not(isLocal) then
        Com_ShutDown;
     TextAttr := 7;
end;

function mCarrier: Boolean;
begin
     if (isLocal) then
        mCarrier := True
     else
     if (Com_Carrier) then
        mCarrier := True
     else
         mCarrier := False;
end;

procedure mClearBuf;
begin
     if Not(isLocal) then
     begin
          Com_PurgeInBuffer;
          Com_PurgeOutBuffer;
     end;
end;

procedure mClrEol;
begin
     mWrite(#27 + '[K');
end;

procedure mClrScr;
begin
     mWrite(#27 + '[2J');
end;

procedure mDisplayFile(S: String);
var
   Sl: TStringList;
   I: Integer;
begin
     if (FileExists(S)) then
     begin
          Sl := TStringList.Create;
          Sl.LoadFromFile(S);
          if (Sl.Count = 1) then
             mWrite(Sl[0])
          else
          if (Sl.Count > 1) then
          begin
               for I := 0 to Sl.Count - 2 do
                   mWriteLn(Sl[I]);
               mWrite(Sl[Sl.Count - 1]);
          end;
          Sl.Free;
     end;
end;

procedure mGotoXY(X, Y: Byte);
begin
     if Not((X < 1) or (X > 80) or (Y < 1) or (Y > 25)) then
        mWrite(#27 + '[' + IntToStr(Y) + ';' + IntToStr(X) + 'f');
end;

procedure mCrlf;
begin
     mWriteLn('');
end;

function mInput(S, Mask: String; ShowLen, MaxLen, Attr: Byte): String;
var
   Ch: Char;
   FG, BG, X: Byte;
begin
     X := WhereX;
     BG := TextAttr div 16;
     FG := TextAttr mod 16;
     mTextBackground(Attr div 16);
     mTextColor(Attr mod 16);

     if (Length(S) > MaxLen) then
        S := Copy(S, 1, MaxLen);

     if (Length(S) > ShowLen) then
        mWrite(Copy(S, Length(S) - ShowLen + 1, ShowLen))
     else
     begin
         mWrite(PadRight(S, ' ', ShowLen));
         mWrite(#27 + '[' + IntToStr(ShowLen - Length(S)) + 'D');
     end;

     repeat
           Ch := mReadKey;
           if (Ch = #0) then
              Ch := mReadKey
           else
           if ((Length(S) < MaxLen) and (Pos(Ch, Mask) > 0)) or ((Length(S) > 0) and (Ch = #8)) then
           begin
                if (Ch = #8) then
                begin
                     Delete(S, Length(S), 1);
                     mWrite(#8#32);
                end else
                    S := S + Ch;

                if (Length(S) >= ShowLen) then
                begin
                     mWrite(#27 + '[' + IntToStr(WhereX - X) + 'D');
                     mWrite(Copy(S, Length(S) - ShowLen + 1, ShowLen));
                end else
                    mWrite(Ch);
           end;
     until (Ch = #13);
     mInput := S;

     mTextBackground(BG);
     mTextColor(FG);
end;

function mInputLn(S, Mask: String; ShowLen, MaxLen, Attr: Byte): String;
begin
     Result := mInput(S, Mask, ShowLen, MaxLen, Attr);
     mCrlf;
end;

function mKeyPressed;
begin
     if (isLocal) then
        mKeyPressed := KeyPressed
     else
     if (KeyPressed) or (Com_CharAvail) then
        mKeyPressed := True
     else
         mKeyPressed := False;
end;

procedure mOpen(ComNum, ComType: LongInt);
begin
     Com_StartUp(ComType);
     Com_SetDontClose(True);
     Com_OpenQuick(ComNum);
end;

function mReadKey: Char;
var
   Ch: Char;
begin
     Ch := #255;
     repeat
           SysCtrlSleep(1);

           if (mKeyPressed) then
           begin
                if (KeyPressed) then
                begin
                     Ch := ReadKey;
                     if (Ch = #0) then
                     begin
                          Ch := ReadKey;
                          isExtended := True;
                     end else
                         isExtended := False;
                     KeyType := ktLocal;
                end else
                if Not(isLocal) then
                begin
                     if (Com_CharAvail) then
                     begin
                          Ch := Com_GetChar;
                          if (Ch = #10) then
                          begin
                               Ch := #255;
                               Continue;
                          end;
                          if (Ch = #0) then
                          begin
                               Ch := ReadKey;
                               isExtended := True;
                          end else
                              isExtended := False;
                          KeyType := ktRemote;
                     end;
                end;
           end;
     until (Ch <> #255);
     mReadKey := Ch;
     LastKeyPress := SecondsSinceMidnight;
end;

procedure mStartUp(DropFile: String; Local: Boolean);
begin
     isLocal := Local;

     if (isLocal) then
        DoLocalLogin
     else
     begin
          DropFile := UpperCase(DropFile);
          if (FileExists(DropFile)) then
          begin
               if (Pos('DORINFO', DropFile) <> 0) then
                  ReadDorinfo(DropFile)
               else
               if (Pos('DOOR32.SYS', DropFile) <> 0) then
                  ReadDoor32(DropFile)
               else
               if (Pos('INFO.', DropFile) <> 0) then
                  ReadInfo(DropFile)
               else
               begin
                    ClrScr;
                    WriteLn;
                    WriteLn('  INVALID DROP FILE TYPE');
                    WriteLn;
                    Delay(2500);
                    Halt;
               end;
          end else
          begin
               ClrScr;
               WriteLn;
               WriteLn('  DROPFILE DOES NOT EXIST');
               WriteLn;
               Delay(2500);
               Halt;
          end;
     end;

     if (mUserInfo.ComNum = 0) or (mUserInfo.ComType = 0) then
        isLocal := True;

     if Not(isLocal) then
     begin
          mOpen(mUserInfo.ComNum, mUserInfo.ComType);

          mClrScr;
          Window(1, 1, 80, 24);

          mUserInfo.TimeOn := SecondsSinceMidnight;
          LastKeyPress := mUserInfo.TimeOn;

          VPBeginThread(HandleProcs, 16384, nil);
     end;
end;

procedure mTextBackground(BG: Byte);
begin
     mWrite(#27 + '[' + IntToStr(40 + COLOURS[BG]) + 'm');
end;

procedure mTextColor(FG: Byte);
begin
     case FG of
          0..7: mWrite(#27 + '[0;' + IntToStr(30 + COLOURS[FG]) + 'm');
         8..15: mWrite(#27 + '[1;' + IntToStr(30 + COLOURS[FG - 8]) + 'm');
     end;
end;

procedure mWrite(S: String);
var
   Count: LongInt;
begin
     MannCodeToAnsi(S);

     if Not(isLocal) then
        Com_SendBlock(S[1], Length(S), Count);
     aWrite(S);
end;

procedure mWriteLn(S: String);
begin
     mWrite(S + #13#10);
end;

initialization
begin
     isLocal := False;

     DropCarrierProc := DefaultDropCarrierProc;
     LocalLoginProc := DefaultLocalLoginProc;
     StatusBarProc := DefaultStatusBarProc;
     TimeOutProc := DefaultTimeOutProc;
     TimeUpProc := DefaultTimeUpProc;

     mUserInfo.IdleMax := 300;
end;

finalization

end.
