unit mGlobal;

interface

uses
    Classes, Crt, MannDoor, MannINI, MannUtls, SockDef, SockFunc, SysUtils;

const
     Copyright = 'MannIRC v1.0';
     Prefix = 'CFC7 ';

var
   ChanList: TStringList;
   CurChan: Integer;
   DoJoin: Boolean;
   DoQuit: Boolean;
   INI: TMannINI;
   IRCServer: String;
   LastCTCP: LongInt;
   ShowMOTD: Boolean;
   Sock: TSockDesc;

procedure BreakUp(S: String; Sl: TStringList);
procedure ClearInput;
function CycleChannel: Boolean;
procedure Disconnect;
procedure DrawInput;
function GetAddress(S: String): String;
function GetNick(S: String): String;
function Join(Sl: TStringList; Count: Integer): String;
function JoinS(Sl: TStringList; Count: Integer): String;
procedure Out(S: String);
procedure OutP(S: String);
procedure RedrawInput;
procedure SockWrite(S: String);
procedure SockWriteLn(S: String);
function Strip(S: String): String;

implementation

uses
    mThreads;

procedure BreakUp(S: String; Sl: TStringList);
begin
     Sl.Clear;

     if (Pos(' ', S) <> 0) then
     begin
          repeat
                Sl.Add(Copy(S, 1, Pos(' ', S) - 1));
                Delete(S, 1, Pos(' ', S));
          until (Pos(' ', S) = 0);
     end;

     if (S <> '') then
        Sl.Add(S);
end;

procedure ClearInput;
begin
     if (WhereX > 1) then
        mWrite(#27 + '[' + IntToStr(WhereX - 1) + 'D' + #27 + '[K');
end;

function CycleChannel: Boolean;
begin
     CycleChannel := False;

     if (ChanList.Count = 1) and (CurChan <> 0) then
     begin
          CycleChannel := True;
          CurChan := 0;
     end else
     if (ChanList.Count > 1) then
     begin
          CycleChannel := True;
          Inc(CurChan);
          if (CurChan > ChanList.Count - 1) then
             CurChan := 1;
     end;
end;

procedure Disconnect;
begin
     ClearInput;
     OutP('Disconnected');
     ChanList.Clear;
     ChanList.Add('Status');
     CurChan := 0;
     SockClose(Sock);
     IRCServer := '<none>';
end;

procedure DrawInput;
var
   MaxLen: Integer;
   S: String;
begin
     S := LocalBuf;
     MaxLen := 79 - Length('[' + ChanList[CurChan] + '] ');
     if (Length(S) > MaxLen) then
        Delete(S, 1, Length(S) - MaxLen);
     mWrite('7[' + ChanList[CurChan] + '] ' + S);
end;

function GetAddress(S: String): String;
var
   Str: String;
   begin
        if (Pos('!', S) = 0) then
           Str := S
        else
            Str := Copy(S, Pos('!', S) + 1, Length(S) - Pos('!', S));

        Result := Str;
end;

function GetNick(S: String): String;
var
   Str: String;
begin
     if (Pos('!', S) = 0) then
        Str := S
     else
         Str := Copy(S, 1, Pos('!', S) - 1);

     Result := Strip(Str);
end;

function Join(Sl: TStringList; Count: Integer): String;
var
   S: String;
   I: Integer;
begin
     S := '';

     if (Count <= Sl.Count - 1) then
     begin
          for I := Count to Sl.Count - 1 do
              S := S + ' ' + Sl[I];
          Delete(S, 1, 1);
     end;

     Result := S;
end;

function JoinS(Sl: TStringList; Count: Integer): String;
var
   S: String;
begin
     S := Join(Sl, Count);
     S := Strip(S);
     JoinS := S;
end;

procedure Out(S: String);
begin
     mWrite('7' + S + #13#10);
end;

{procedure Out(S: String);
var
   I: Integer;
   Str: String;
   First: Boolean;
   DoPadRight: Boolean;

  procedure DoWrite(S: String);
  begin
       S := '7' + S;
       if (First) then
          mWrite(S)
       else
           mWrite('    ' + S);
       if ((First) and (Length(S) < 80)) or (Not(First) and (Length(S) < 76)) then
          mCrlf;
       First := False;
  end;

begin
     First := True;

     if (Length(S) <= 80) then
        DoWrite(S)
     else
     begin
          repeat
                if (First) then
                   I := 80
                else
                    I := 76;
                Str := TrimLeft(Copy(S, 1, I));
                Delete(S, 1, I);

                if (Str[Length(Str)] = ' ') then
                   DoPadRight := True
                else
                    DoPadRight := False;

                if (Pos(' ', Str) = 0) then
                   DoWrite(Str)
                else
                begin
                     for I := Length(Str) downto 1 do
                         if (Str[I] = ' ') then
                            Break;
                     if (I = 4) then
                        DoWrite(Str)
                     else
                     begin
                          DoWrite(Copy(Str, 1, I));
                          Delete(Str, 1, I);
                          if (DoPadRight) then
                             S := Str + ' ' + S
                          else
                              S := Str + S;
                     end;
                end;
          until (Length(S) <= 76);
          if (Trim(S) <> '') then
             DoWrite(Trim(S));
     end;
end;}

procedure OutP(S: String);
begin
     Out(Prefix + S);
end;

procedure RedrawInput;
begin
     ClearInput;
     DrawInput;
end;

procedure SockWrite(S: String);
begin
     SockSend(Sock, @S[1], Length(S), 0);
end;

procedure SockWriteLn(S: String);
begin
     SockWrite(S + #13#10);
end;

function Strip(S: String): String;
begin
     if (S <> '') then
        if (S[1] = ':') then
           Delete(S, 1, 1);
     Strip := S;
end;

initialization
begin
     ChanList := TStringList.Create;
     ChanList.Add('Status');
     CurChan := 0;
     DoJoin := False;
     DoQuit := False;
     Ini := TMannINI.Create(ExtractFilePath(ParamStr(0)) + 'CONFIG.INI');
     IRCServer := '<none>';
     LastCTCP := 0;
     ShowMOTD := False;
end;

finalization
begin
     ChanList.Free;
     Ini.Free;
end;

end.
